﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Collections.ObjectModel;
using System.Windows.Resources;
using System.IO;
using System.ComponentModel;
using Infragistics.Documents.RichText;
using Infragistics;
using Infragistics.Documents.RichText.Serialization;
using Infragistics.Documents.RichText.Word;
using Infragistics.Controls.Editors;
using Infragistics.Controls.Editors.Primitives;

#if !SILVERLIGHT
using Microsoft.Win32;
#endif

namespace XamRichTextEditorStudio
{
	/// <summary>
	/// This UserControl contains a XamRichTextEditor and a XamRibbon control to simulate the look and behavior of a
	/// typical rich text editing application.  
	/// 
	/// As a result, this class contains a lot of code that relates to performing basic application tasks 
	/// (e.g., displaying a menu bar for exposing various control commands,  Opening/Saving files etc)
	/// and not to directly manipulating the XamRichTextEditor.  While this code is provided to show how the XamRichTextEditor could
	/// be used in a real world editing application, it has not been designed and debugged for use in a production application.
	/// It is highly suggested that you use this code as a reference only and not directly in a production application.
	/// 
	/// This UserControl is designed to be used in both Silverlight and WPF.  As a result you will notice #ifdefs sprinkled 
	/// throughout the code to account for the differences between SL & WPF (none directly relating to the XamRichTextEditor),
	/// primarily in the area of file handling which is significantly different in SL vs WPF due to security restrictions in SL.
	/// </summary>
	public partial class RichTextEditorStudioUserControl : UserControl, INotifyPropertyChanged
	{
		#region Member Variables

		private string								_windowTitle;
		private string								_currentFilenameShort;
		private string								_currentFilenameLong;
		private FileInfo							_currentFileInfo;
		private bool								_hasUnsavedChanges;
		private bool								_hasCurrentDocument;

#if WPF
		private bool								_currentFileIsNew;
#endif

		private Brush								_statusBarBackgroundBrush;
		private Brush								_statusBarForegroundBrush;

		private Dictionary<string, DocumentFormatInfo>
													_supportedFormats;

		private DocumentFormatInfo					_currentDocumentFormatInfo;

		#endregion //Member Variables

		#region Constants

		private static readonly string WINDOW_TITLE				= "XamRichTextEditorStudio [{0}]";
		private static readonly string NEW_DOCUMENT_LITERAL		= "UNTITLED";

		#endregion //Constants

		#region Constructor
		public RichTextEditorStudioUserControl()
		{
			InitializeComponent();

			this.Initialize();
		}
		#endregion //Constructor

		#region Properties

		#region Public Properties

		#region CurrentCharacterSettings
		/// <summary>
		/// Returns the CharacterSettings used by the content represented by the current selection in the active view.
		/// </summary>
		public CharacterSettings CurrentCharacterSettings
		{
			get { return this.CurrentDocument.GetCommonCharacterSettings(this.GetCurrentSelectionDocumentSpan()); }
		}
		#endregion //CurrentCharacterSettings

		#region CurrentDocument
		/// <summary>
		/// Returns/sets the document currently being edited by the XamRichTextEditor.
		/// </summary>
		/// <seealso cref="DocumentStatisticsText"/>
		public RichTextDocument CurrentDocument
		{
			get { return this.xamRichTextEditor1.Document; }
			set
			{
				this.xamRichTextEditor1.Document = value;
				if (this.PropertyChanged != null)
					this.PropertyChanged(this, new PropertyChangedEventArgs("CurrentDocument"));
			}
		}
		#endregion // CurrentDocument

		#region CurrentParagraphSettings
		/// <summary>
		/// Returns the ParagraphSettings used by the paragraphs that intersect the current selection in the active view.
		/// </summary>
		public ParagraphSettings CurrentParagraphSettings
		{
			get { return this.CurrentDocument.GetCommonParagraphSettings(this.GetCurrentSelectionDocumentSpan()); }
		}
		#endregion //CurrentParagraphSettings

		#region FontIsBoldInCurrentSelection
		public static readonly DependencyProperty FontIsBoldInCurrentSelectionProperty =
			DependencyProperty.Register("FontIsBoldInCurrentSelection", typeof(bool),
			typeof(RichTextEditorStudioUserControl), null);

		/// <summary>
		/// Returns whether the font used by the content represented by the current selection in the active view is bold.
		/// </summary>
		public bool FontIsBoldInCurrentSelection
		{
			get { return (bool)GetValue(FontIsBoldInCurrentSelectionProperty); }
			private set { SetValue(FontIsBoldInCurrentSelectionProperty, value); }
		}
		#endregion //FontIsBoldInCurrentSelection

		#region FontIsItalicInCurrentSelection
		public static readonly DependencyProperty FontIsItalicInCurrentSelectionProperty =
			DependencyProperty.Register("FontIsItalicInCurrentSelection", typeof(bool),
			typeof(RichTextEditorStudioUserControl), null);

		/// <summary>
		/// Returns whether the font used by the content represented by the current selection in the active view is italicized.
		/// </summary>
		public bool FontIsItalicInCurrentSelection
		{
			get { return (bool)GetValue(FontIsItalicInCurrentSelectionProperty); }
			private set { SetValue(FontIsItalicInCurrentSelectionProperty, value); }
		}
		#endregion //FontIsItalicInCurrentSelection

		#region FontIsSingleStrikethroughInCurrentSelection
		public static readonly DependencyProperty FontIsSingleStrikethroughInCurrentSelectionProperty =
			DependencyProperty.Register("FontIsSingleStrikethroughInCurrentSelection", typeof(bool),
			typeof(RichTextEditorStudioUserControl), null);

		/// <summary>
		/// Returns whether the font used by the content represented by the current selection in the active view is italicized.
		/// </summary>
		public bool FontIsSingleStrikethroughInCurrentSelection
		{
			get { return (bool)GetValue(FontIsSingleStrikethroughInCurrentSelectionProperty); }
			private set { SetValue(FontIsSingleStrikethroughInCurrentSelectionProperty, value); }
		}
		#endregion //FontIsSingleStrikethroughInCurrentSelection

		#region FontNameInCurrentSelection
		public static readonly DependencyProperty FontNameInCurrentSelectionProperty =
			DependencyProperty.Register("FontNameInCurrentSelection", typeof(string),
			typeof(RichTextEditorStudioUserControl), null);

		/// <summary>
		/// Returns the name of the font used by the content represented by the current selection in the active view.
		/// </summary>
		/// <remarks>
		/// This property will return string.Empty if the current selection includes content with multiple font settings or if there is no active view.
		/// </remarks>
		public string FontNameInCurrentSelection
		{
			get { return (string)GetValue(FontNameInCurrentSelectionProperty); }
			private set { SetValue(FontNameInCurrentSelectionProperty, value); }
		}
		#endregion //FontNameInCurrentSelection

		#region FontSizeInCurrentSelection
		public static readonly DependencyProperty FontSizeInCurrentSelectionProperty =
			DependencyProperty.Register("FontSizeInCurrentSelection", typeof(Extent?),
			typeof(RichTextEditorStudioUserControl), null);

		/// <summary>
		/// Returns the size of the font used by the content represented by the current selection in the active view.
		/// </summary>
		/// <remarks>
		/// This property will return null if the current selection includes content with multiple font settings or if there is no active view.
		/// </remarks>
		public Extent? FontSizeInCurrentSelection
		{
			get { return (Extent?)GetValue(FontSizeInCurrentSelectionProperty); }
			private set { SetValue(FontSizeInCurrentSelectionProperty, value); }
		}
		#endregion //FontSizeInCurrentSelection

		#region HasCurrentDocument
		/// <summary>
		/// Returns whether a document is currently set on the XamRichTextEditor.
		/// </summary>
		public bool HasCurrentDocument
		{
			get { return this._hasCurrentDocument; }
			private set 
			{
				if (value != this._hasCurrentDocument)
				{
					this._hasCurrentDocument = value;
					if (this.PropertyChanged != null)
						this.PropertyChanged(this, new PropertyChangedEventArgs("HasCurrentDocument"));
				}
			}
		}
		#endregion //HasCurrentDocument

		#region HasUnsavedChanges (read only)
		/// <summary>
		/// Returns true if the document has unsaved changes. (read only)
		/// </summary>
		public bool HasUnsavedChanges
		{
			get { return this._hasUnsavedChanges; }
			private set
			{
				this._hasUnsavedChanges = value;
				this.UpdateWindowTitle();
				this.RaisePropertyChanged("HasUnsavedChanges");
			}
		}
		#endregion //HasUnsavedChanges

		#region ParagraphIsAlignedLeftInCurrentSelection
		public static readonly DependencyProperty ParagraphIsAlignedLeftInCurrentSelectionProperty =
			DependencyProperty.Register("ParagraphIsAlignedLeftInCurrentSelection", typeof(bool),
			typeof(RichTextEditorStudioUserControl), null);

		/// <summary>
		/// Returns whether the paragraph(s) that inersect the current selection in the active view are aligned left.
		/// </summary>
		public bool ParagraphIsAlignedLeftInCurrentSelection
		{
			get { return (bool)GetValue(ParagraphIsAlignedLeftInCurrentSelectionProperty); }
			private set { SetValue(ParagraphIsAlignedLeftInCurrentSelectionProperty, value); }
		}
		#endregion //ParagraphIsAlignedLeftInCurrentSelection

		#region ParagraphIsAlignedCenterInCurrentSelection
		public static readonly DependencyProperty ParagraphIsAlignedCenterInCurrentSelectionProperty =
			DependencyProperty.Register("ParagraphIsAlignedCenterInCurrentSelection", typeof(bool),
			typeof(RichTextEditorStudioUserControl), null);

		/// <summary>
		/// Returns whether the paragraph(s) that inersect the current selection in the active view are aligned center.
		/// </summary>
		public bool ParagraphIsAlignedCenterInCurrentSelection
		{
			get { return (bool)GetValue(ParagraphIsAlignedCenterInCurrentSelectionProperty); }
			private set { SetValue(ParagraphIsAlignedCenterInCurrentSelectionProperty, value); }
		}
		#endregion //ParagraphIsAlignedCenterInCurrentSelection

		#region ParagraphIsAlignedRightInCurrentSelection
		public static readonly DependencyProperty ParagraphIsAlignedRightInCurrentSelectionProperty =
			DependencyProperty.Register("ParagraphIsAlignedRightInCurrentSelection", typeof(bool),
			typeof(RichTextEditorStudioUserControl), null);

		/// <summary>
		/// Returns whether the paragraph(s) that inersect the current selection in the active view are aligned right.
		/// </summary>
		public bool ParagraphIsAlignedRightInCurrentSelection
		{
			get { return (bool)GetValue(ParagraphIsAlignedRightInCurrentSelectionProperty); }
			private set { SetValue(ParagraphIsAlignedRightInCurrentSelectionProperty, value); }
		}
		#endregion //ParagraphIsAlignedRightInCurrentSelection

		#region ParagraphIsAlignedJustifiedInCurrentSelection
		public static readonly DependencyProperty ParagraphIsAlignedJustifiedInCurrentSelectionProperty =
			DependencyProperty.Register("ParagraphIsAlignedJustifiedInCurrentSelection", typeof(bool),
			typeof(RichTextEditorStudioUserControl), null);

		/// <summary>
		/// Returns whether the paragraph(s) that inersect the current selection in the active view are aligned justified.
		/// </summary>
		public bool ParagraphIsAlignedJustifiedInCurrentSelection
		{
			get { return (bool)GetValue(ParagraphIsAlignedJustifiedInCurrentSelectionProperty); }
			private set { SetValue(ParagraphIsAlignedJustifiedInCurrentSelectionProperty, value); }
		}
		#endregion //ParagraphIsAlignedJustifiedInCurrentSelection

		#region RichTextEditor
		/// <summary>
		/// Returns the XamRichTextEditor control hosted in this user control.
		/// </summary>
		public XamRichTextEditor RichTextEditor
		{
			get { return this.xamRichTextEditor1; }
		}
		#endregion //RichTextEditor

		#region StatusBarBackgroundBrush
		/// <summary>
		/// Returns/sets the brush used to draw the background of status bar area.
		/// </summary>
		public Brush StatusBarBackgroundBrush
		{
			get { return this._statusBarBackgroundBrush; }
			set
			{
				this._statusBarBackgroundBrush = value;
				this.RaisePropertyChanged("StatusBarBackgroundBrush");
			}
		}
		#endregion //StatusBarBackgroundBrush

		#region StatusBarForegroundBrush
		/// <summary>
		/// Returns/sets the brush used to draw the foreground of the elements in the status bar area.
		/// </summary>
		public Brush StatusBarForegroundBrush
		{
			get { return this._statusBarForegroundBrush; }
			set
			{
				this._statusBarForegroundBrush = value;
				this.RaisePropertyChanged("StatusBarForegroundBrush");
			}
		}
		#endregion //StatusBarForegroundBrush

		#endregion //Public Properties

		#region Public Read Only Properties

		#region WindowTitle (read only)
		/// <summary>
		/// Returns the Window title text. (read only)
		/// </summary>
		/// <seealso cref="UpdateWindowTitle"/>
		public string WindowTitle
		{
			get { return this._windowTitle; }
			private set
			{
				if (this._windowTitle != value)
				{
					this._windowTitle = value;
					this.RaisePropertyChanged("WindowTitle");
				}
			}
		}
		#endregion //WindowTitle (read only)

		#endregion // Public Read Only Properties

		#region Private Properties

		#region SupportedDocumentFormats (read only)
		/// <summary>
		/// Returns a dictionary (keyed by file extension) of all the document formats supported by the sample. (read only)
		/// </summary>
		private Dictionary<string, DocumentFormatInfo> SupportedDocumentFormats
		{
			get
			{
				if (this._supportedFormats == null)
				{
					this._supportedFormats = new Dictionary<string, DocumentFormatInfo>(2, StringComparer.CurrentCultureIgnoreCase);

					this._supportedFormats.Add(".docx", new DocumentFormatInfo(new WordSerializationProvider(), ".docx", "An MS Word document in docx format"));
					this._supportedFormats.Add(".txt", new DocumentFormatInfo(new PlainTextSerializationProvider(), ".txt", "An plain text document"));
				}

				return this._supportedFormats;
			}
		}
		#endregion //SupportedFormats (read only)

		#endregion //Private Properties

		#endregion //Properties

		#region Methods

		#region Public Methods

		#region CloseDocument
		public void CloseDocument()
		{
			// If the current document has unsaved changes, prompt to save it.
			if (this.HasUnsavedChanges)
			{
				MessageBoxResult result;
#if WPF
				bool cancel = this.PromptForSave();
				if (cancel)
					return;
#else
				result = MessageBox.Show("The current document has unsaved changes.  Click OK to discard the changes and create a new empty document or Cancel to return to the document without loading the empty document.", "XamRichTextEditorStudio", MessageBoxButton.OKCancel);
				if (result == MessageBoxResult.Cancel)
					return;
#endif
			}

			this.ClearFindResultsAndErrorLists();

			this._currentDocumentFormatInfo		= null;
			this.xamRichTextEditor1.Document	= null;

			this.NewDocument( );
		}
		#endregion //CloseDocument

		#region HighlightColorFromColor (static)
		public static HighlightColor HighlightColorFromColor(Color color)
		{
			if (color == Colors.Black)
				return HighlightColor.Black;
			if (color == Colors.Blue)
				return HighlightColor.Blue;
			if (color == Colors.Cyan)
				return HighlightColor.Cyan;
			if (color == Color.FromArgb(255, 0, 0, 128))
				return HighlightColor.DarkBlue;
			if (color == Color.FromArgb(255, 0, 128, 128))
				return HighlightColor.DarkCyan;
			if (color == Colors.DarkGray)
				return HighlightColor.DarkGray;
			if (color == Color.FromArgb(255, 0, 128, 0))
				return HighlightColor.DarkGreen;
			if (color == Color.FromArgb(255, 128, 0, 128))
				return HighlightColor.DarkMagenta;
			if (color == Color.FromArgb(255, 128, 0, 0))
				return HighlightColor.DarkRed;
			if (color == Color.FromArgb(255, 128, 128, 00))
				return HighlightColor.DarkYellow;
			if (color == Colors.Green)
				return HighlightColor.Green;
			if (color == Colors.LightGray)
				return HighlightColor.LightGray;
			if (color == Colors.Magenta)
				return HighlightColor.Magenta;
			if (color == Colors.Red)
				return HighlightColor.Red;
			if (color == Colors.White)
				return HighlightColor.White;
			if (color == Colors.Yellow)
				return HighlightColor.Yellow;

			return HighlightColor.Yellow;
		}
		#endregion //HighlightColorFromColor (static)

		#region NewDocument
		public void NewDocument()
		{
			// Load an empty docx document.
			this.LoadEmptyDocument(this.SupportedDocumentFormats[".docx"]);
		}
		#endregion //NewDocument

		#region OpenDocument
#if SILVERLIGHT
		public void OpenDocument()
		{
			// Show a file open dialog.
			OpenFileDialog dialog	= new OpenFileDialog();
			dialog.FilterIndex		= 1;
			dialog.Filter			= "*.docx Files|*.docx|Text documents|*.txt";

			bool? open				= dialog.ShowDialog();
			if (open.HasValue && open.Value == true)
			{
				// If the current document is dirty, prompt the user.
				if (this.HasUnsavedChanges)
				{
					MessageBoxResult result = MessageBox.Show("The current document has unsaved changes.  Click OK to discard the changes and open the new file or Cancel to return to the document without opening the file.", "XamRichTextEditorStudio", MessageBoxButton.OKCancel);
					if (result == MessageBoxResult.Cancel)
						return;
					else
						this.HasUnsavedChanges = false;
				}

				this._currentFileInfo		= dialog.File;
				this._currentFilenameShort	= dialog.File.Name;
				this._currentFilenameLong	= dialog.File.Name;

				// Create a TextDocument with a format based on the file extension.
				string ext = System.IO.Path.GetExtension(this._currentFilenameShort);
				if (this.SupportedDocumentFormats.ContainsKey(ext))
					this._currentDocumentFormatInfo = this.SupportedDocumentFormats[ext];
				else
				{
					MessageBox.Show(string.Format("Files with extension '{0}' are not currently supported by XamRichTextEditor.", ext), "Error Loading File", MessageBoxButton.OK);
					return;
				}

				RichTextDocument document = new RichTextDocument();

				this.ClearFindResultsAndErrorLists();

				// Load the file into the RichTextDocument.
				FileStream fileStream = null;
				try
				{
					fileStream = this._currentFileInfo.OpenRead();
					document.Load(this._currentDocumentFormatInfo.SerializationProvider, fileStream);
				}
				catch (Exception ex)
				{
					MessageBox.Show(string.Format("Error '{0}' while loading file '{1}'", "Error Loading File", ex.Message, this._currentFilenameLong), "Error Loading File", MessageBoxButton.OK);
					return;
				}
				finally
				{
					if (null != fileStream)
						fileStream.Close();
				}

				// Assign the RichTextDocument to the XamRichTextEditor
				this.xamRichTextEditor1.Document = document;

				// Update the window title to reflect the new document name.
				this.UpdateWindowTitle();
			}
		}
#else
		public void OpenDocument()
		{
			// If the current document has unsaved changes, prompt to save it.
			if (this.HasUnsavedChanges)
			{
				bool cancel = this.PromptForSave();
				if (cancel)
				    return;
			}

			// Show a file open dialog.
			string fileName			= "";
			OpenFileDialog dialog	= new OpenFileDialog();

			dialog.FilterIndex		= 1;
			dialog.Title			= "Open File";
			dialog.DefaultExt		= ".docx";
			dialog.FileName			= fileName;
			dialog.Filter =			"docx Files|*.docx|Text documents|*.txt";

			bool? open				= dialog.ShowDialog();
			if (open.HasValue && open.Value == true)
			{
				// Save information about the file.
				this._currentFilenameShort	= dialog.SafeFileName;
				this._currentFilenameLong	= dialog.FileName;
				this._currentFileInfo		= new FileInfo(this._currentFilenameLong);

				this._currentFileIsNew		= false;

				// Create a TextDocument with a format based on the file extension.
				string ext = System.IO.Path.GetExtension(this._currentFilenameShort);
				if (this.SupportedDocumentFormats.ContainsKey(ext))
					this._currentDocumentFormatInfo = this.SupportedDocumentFormats[ext];
				else
				{
					MessageBox.Show(string.Format("Files with extension '{0}' are not currently supported by XamRichTextEditor.", ext), "Error Loading File", MessageBoxButton.OK);
					return;
				}

				RichTextDocument document = new RichTextDocument();

				this.ClearFindResultsAndErrorLists();

				// Load the file into the RichTextDocument.
				FileStream fileStream = null;
				try
				{
					fileStream = this._currentFileInfo.OpenRead();
					document.Load(this._currentDocumentFormatInfo.SerializationProvider, fileStream);
				}
				catch (Exception ex)
				{
					MessageBox.Show(string.Format("Error '{0}' while loading file '{1}'", ex.Message, this._currentFilenameLong), "Error Loading File", MessageBoxButton.OK);
					return;
				}
				finally
				{
					if (null != fileStream)
						fileStream.Close();
				}

				// Assign the RichTextDocument to the XamRichTextEditor.
				this.xamRichTextEditor1.Document = document;

				// Update the window title to reflect the new document name.
				this.UpdateWindowTitle();
			}
		}
#endif
		#endregion //OpenDocument

		#region SaveDocument
		public void SaveDocument()
		{
			this.SaveCurrentDocument();
		}
		#endregion //SaveDocument

		#region UnderlineTypeFromDescription (static)
		/// <summary>
		/// Returns an UnderlineType from a descriptive string.
		/// </summary>
		/// <param name="toolCaption"></param>
		/// <returns></returns>
		public static UnderlineType UnderlineTypeFromDescription(string toolCaption)
		{
			switch (toolCaption)
			{
				case "Dash":
					return UnderlineType.Dash;
				case "Single":
					return UnderlineType.Single;
				case "SingleHeavy":
					return UnderlineType.Thick;
				case "Double":
					return UnderlineType.Double;
				case "DashLong":
					return UnderlineType.DashLong;
				case "DashLong Heavy":
					return UnderlineType.DashLongHeavy;
				case "Dash-Dot-Dot Heavy":
					return UnderlineType.DashDotDotHeavy;
				case "Dash-Dot Heavy":
					return UnderlineType.DashDotHeavy;
				case "Dashed Heavy":
					return UnderlineType.DashedHeavy;
				case "Dot-Dash":
					return UnderlineType.DotDash;
				case "Dot-Dot-Dash":
					return UnderlineType.DotDotDash;
				case "Dotted":
					return UnderlineType.Dotted;
				case "Dotted Heavy":
					return UnderlineType.DottedHeavy;
				default:
					return UnderlineType.None;
			}
			return UnderlineType.None;
		}
		#endregion //UnderlineTypeFromDescription (static)

		#endregion //Public Methods

		#region Private Methods

		#region CheckForUnsupportedContent
		/// <summary>
		/// Checks to see if the documetn has unsupported content and if so, prompts the user to ask if they want to continue.
		/// </summary>
		/// <returns>True if the operation that triggered this prompt should be canceled.</returns>
		private bool CheckForUnsupportedContent()
		{
			if (this.CurrentDocument.RootNode.HasUnsupportedContent)
			{
				MessageBoxResult	result;
				string				message = "This document contains content that unsupported in this April CTP of the XamRichTextEditor control. If you save this document all unsupported content will be lost (refer to the release notes for the April CTP for more information on what contnet is supported).  Are you sure you want to save the document and lose this content?";
#if SILVERLIGHT
				result = MessageBox.Show(message, "XamRichTextEditorStudio", MessageBoxButton.OKCancel);
#else
				result = MessageBox.Show(message, "XamRichTextEditorStudio", MessageBoxButton.OKCancel, MessageBoxImage.Stop, MessageBoxResult.Cancel);
#endif
				if (result == MessageBoxResult.Cancel)
					return true;
			}

			return false;
		}
		#endregion //CheckForUnsupportedContent

		#region ClearFindResultsAndErrorLists
		private void ClearFindResultsAndErrorLists()
		{
		}
		#endregion //ClearFindResultsAndErrorLists

		#region GetCurrentSelectionDocumentSpan
		private DocumentSpan GetCurrentSelectionDocumentSpan()
		{
			DocumentSpan	documentSpan	= new DocumentSpan(0, 0);
			Selection		selection		= this.xamRichTextEditor1.Selection;
			if (null != selection)
				documentSpan = selection.DocumentSpan;

			return documentSpan;
		}
		#endregion //GetCurrentSelectionDocumentSpan

		#region Initialize
		/// <summary>
		/// Performs basic initialization of the XamRichTextEditorUserControl.
		/// </summary>
		private void Initialize()
		{
			// Load an empty docx document.
			this.LoadEmptyDocument(this.SupportedDocumentFormats[".docx"]);

			// Wire XamRichTextEditor event listeners.
			this.WireRichTextEditorEventHandlers();

			// Initialize RichTextEditorStudioUserControl colors to defaults.
			this.SetStudioColorScheme("DEFAULT");
		}
		#endregion //Initialize

		#region LoadEmptyDocument
		/// <summary>
		/// Creates an empty document and assigns it to the XamRichTextEditor.Document property.
		/// </summary>
		private void LoadEmptyDocument(DocumentFormatInfo documentFormat)
		{
			if (this.HasUnsavedChanges)
			{
				MessageBoxResult result;
#if WPF
				bool cancel = this.PromptForSave();
				if (cancel)
					return;
#else
				result = MessageBox.Show("The current document has unsaved changes.  Click OK to discard the changes and create a new empty document or Cancel to return to the document without loading the empty document.", "XamRichTextEditorStudio", MessageBoxButton.OKCancel);
				if (result == MessageBoxResult.Cancel)
					return;
#endif
			}

			this.ClearFindResultsAndErrorLists();

			RichTextDocument document			= new RichTextDocument();
			this._currentDocumentFormatInfo		= documentFormat;
			this.xamRichTextEditor1.Document	= document;
			

#if WPF
			this._currentFileIsNew				= true;
#endif
		}
		#endregion //LoadEmptyDocument

		#region PromptForSave
#if WPF
		/// <summary>
		/// Prompts the user to save the current document, and then saves the document if the user responds 'yes'. (WPF only)
		/// </summary>
		/// <returns>True if the operation that triggered this prompt should be canceled.</returns>
		private bool PromptForSave()
		{
			if (true == this.CheckForUnsupportedContent())
				return true;

			string fileName;

			if (this._currentFileIsNew)
				fileName = NEW_DOCUMENT_LITERAL;
			else
				fileName = this._currentFilenameShort;

			MessageBoxResult	result;
			result = MessageBox.Show(string.Format("Do you want to save changes to '{0}'?", fileName), "XamRichTextEditorStudio", MessageBoxButton.YesNoCancel);
			if (result == MessageBoxResult.Cancel)
				return true;

			if (result == MessageBoxResult.Yes)
				this.SaveCurrentDocument();

			return false;
		}
#endif
		#endregion //PromptForSave

		#region RaisePropertyChanged
		/// <summary>
		/// Raises the PropertyChanged event for the specified property if the event has listeners attached.
		/// </summary>
		/// <param name="propertyName"></param>
		private void RaisePropertyChanged(string propertyName)
		{
			if (this.PropertyChanged != null)
				this.PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
		}
		#endregion //RaisePropertyChanged

		#region SaveCurrentDocument
#if WPF
		/// <summary>
		/// Saves the current document. (WPF only)
		/// </summary>
		private void SaveCurrentDocument()
		{
			if ( null == this.CurrentDocument )
				return;

			if (true == this.CheckForUnsupportedContent())
				return;

			if (this._currentFileIsNew)
				this.SaveNewDocument();
			else
				this.WriteCurrentDocumentContentsToFileStreamAndCloseStream(this._currentFileInfo.Open(FileMode.Create, FileAccess.ReadWrite, FileShare.None), true);
		}
#else
		/// <summary>
		/// Displays a SaveFileDialog to save the current document. (Silverlight only)
		/// </summary>
		private void SaveCurrentDocument()
		{
			if (true == this.CheckForUnsupportedContent())
				return;

			this.SaveNewDocument();
		}
#endif
		#endregion //SaveCurrentDocument

		#region SaveNewDocument
		/// <summary>
		/// Saves a 'new' document (i.e. one that was not originally loaded from a file).
		/// </summary>
		/// <returns></returns>
		private bool SaveNewDocument()
		{
			SaveFileDialog saveDialog	= new SaveFileDialog();
			saveDialog.DefaultExt		= ".docx";
			saveDialog.Filter			= "*.docx Files|*.docx|Text documents|*.txt";

			bool? result = saveDialog.ShowDialog();
			if (result == true)
			{
				this._currentFilenameShort = saveDialog.SafeFileName;
#if WPF
				this._currentFilenameLong = saveDialog.FileName;
				this._currentFileIsNew = false;
				this._currentFileInfo = new FileInfo(this._currentFilenameLong);
#else
				this._currentFilenameLong = saveDialog.SafeFileName;
#endif

				string ext = System.IO.Path.GetExtension(this._currentFilenameShort);
				if (this.SupportedDocumentFormats.ContainsKey(ext))
					this._currentDocumentFormatInfo = this.SupportedDocumentFormats[ext];
				else
				{
					MessageBox.Show(string.Format("Files with extension '{0}' are not currently supported by XamRichTextEditor.", ext), "Error Loading File", MessageBoxButton.OK);
					return false;
				}

				this.WriteCurrentDocumentContentsToFileStreamAndCloseStream(saveDialog.OpenFile(), true);
			}

			return false;
		}
		#endregion //SaveNewDocument

		#region SetStudioColorScheme
		private void SetStudioColorScheme(string colorSchemeName)
		{
			this.StatusBarBackgroundBrush = this.Resources[colorSchemeName + "_StatusBarBackgroundBrush"] as Brush;
			this.StatusBarForegroundBrush = this.Resources[colorSchemeName + "_StatusBarForegroundBrush"] as Brush;
		}
		#endregion //SetStudioColorScheme

		#region UpdateSettingsProperties
		private void UpdateSettingsProperties()
		{
			if (null == this.CurrentDocument)
				return;

			CharacterSettings characterSettings = this.CurrentCharacterSettings;

			// Update the state of the FontIsBoldInCurrentSelection property.
			if (false == characterSettings.Bold.HasValue || false == characterSettings.Bold.Value)
				this.FontIsBoldInCurrentSelection = false;
			else
				this.FontIsBoldInCurrentSelection = true;

			// Update the state of the FontIsItalicInCurrentSelection property.
			if (false == characterSettings.Italics.HasValue || false == characterSettings.Italics.Value)
				this.FontIsItalicInCurrentSelection = false;
			else
				this.FontIsItalicInCurrentSelection = true;

			// Update the state of the FontIsSingleStrikethroughInCurrentSelection property.
			if (false == characterSettings.StrikeThrough.HasValue || false == characterSettings.StrikeThrough.Value)
				this.FontIsSingleStrikethroughInCurrentSelection = false;
			else
				this.FontIsSingleStrikethroughInCurrentSelection = true;

			ParagraphSettings paragraphSettings = this.CurrentParagraphSettings;

			// Update the state of the ParagraphIsAlignedLeftInCurrentSelection property.
			if (false == paragraphSettings.ParagraphAlignment.HasValue || ParagraphAlignment.Start != paragraphSettings.ParagraphAlignment.Value)
				this.ParagraphIsAlignedLeftInCurrentSelection = false;
			else
				this.ParagraphIsAlignedLeftInCurrentSelection = true;

			// Update the state of the ParagraphIsAlignedCenterInCurrentSelection property.
			if (false == paragraphSettings.ParagraphAlignment.HasValue || ParagraphAlignment.Center != paragraphSettings.ParagraphAlignment.Value)
				this.ParagraphIsAlignedCenterInCurrentSelection = false;
			else
				this.ParagraphIsAlignedCenterInCurrentSelection = true;

			// Update the state of the ParagraphIsAlignedRightInCurrentSelection property.
			if (false == paragraphSettings.ParagraphAlignment.HasValue || ParagraphAlignment.End != paragraphSettings.ParagraphAlignment.Value)
				this.ParagraphIsAlignedRightInCurrentSelection = false;
			else
				this.ParagraphIsAlignedRightInCurrentSelection = true;

			// Update the state of the ParagraphIsAlignedJustifiedInCurrentSelection property.
			if (false == paragraphSettings.ParagraphAlignment.HasValue || ParagraphAlignment.Justify != paragraphSettings.ParagraphAlignment.Value)
				this.ParagraphIsAlignedJustifiedInCurrentSelection = false;
			else
				this.ParagraphIsAlignedJustifiedInCurrentSelection = true;
		}
		#endregion //UpdateSettingsProperties

		#region UpdateWindowTitle
		/// <summary>
		/// Updates the RichTextEditorStudioUserControl's <see cref="WindowTitle"/> property to reflect the name of the current document. 
		/// </summary>
		/// <seealso cref="WindowTitle"/>
		private void UpdateWindowTitle()
		{
			string title = string.Empty;

			if (string.IsNullOrEmpty(this._currentFilenameShort))
				title = string.Format(WINDOW_TITLE, NEW_DOCUMENT_LITERAL);
			else
				title = string.Format(WINDOW_TITLE, this._currentFilenameShort);

			if (this.HasUnsavedChanges)
				title += "*";

			this.WindowTitle = title;
		}
		#endregion //UpdateWindowTitle

		#region WireRichTextEditorEventHandlers
		/// <summary>
		/// Wires event handlers for the XamRichTextEditor events we are interested in.
		/// </summary>
		private void WireRichTextEditorEventHandlers()
		{
			this.xamRichTextEditor1.DocumentChanged			+= xamRichTextEditor1_DocumentChanged;
			this.xamRichTextEditor1.DocumentContentChanged	+= xamRichTextEditor1_DocumentContentChanged;
			this.xamRichTextEditor1.SelectionChanged		+= xamRichTextEditor1_SelectionChanged;
		}
		#endregion //WireRichTextEditorEventHandlers

		#region WriteCurrentDocumentContentsToFileStreamAndCloseStream
		/// <summary>
		/// Write the current document contents to the specified stream and optionally close the stream. 
		/// </summary>
		/// <param name="fileStream">The Stream to write to</param>
		private void WriteCurrentDocumentContentsToFileStreamAndCloseStream(Stream fileStream, bool close)
		{
			try
			{
				if (_currentDocumentFormatInfo != null)
					this.xamRichTextEditor1.Document.Save(_currentDocumentFormatInfo.SerializationProvider, fileStream);
				else
					this.xamRichTextEditor1.Document.SaveToWord(fileStream);
			}
			catch (Exception ex)
			{
				MessageBox.Show(string.Format("Error '{0}' while saving file '{1}'", ex.Message, this._currentFileInfo.Name), "Error Saving File", MessageBoxButton.OK);
			}
			finally
			{
				if (close)
					fileStream.Close();
			}

			this.HasUnsavedChanges = false;
		}
		#endregion //WriteCurrentDocumentContentsToFileStreamAndCloseStream

		#endregion //Private Methods

		#endregion //Methods

		#region Event Handlers

		#region RichTextDocument Event Handlers

		#endregion //RichTextDocument Event Handlers

		#region RichTextEditorStudioUserControl Control Event Handlers

		#region ZoomIn_Click
		private void ZoomIn_Click(object sender, RoutedEventArgs e)
		{
			this.slider.Value += this.slider.SmallChange;
		}
		#endregion //ZoomIn_Click

		#region ZoomOut_Click
		private void ZoomOut_Click(object sender, RoutedEventArgs e)
		{
			this.slider.Value -= this.slider.SmallChange;
		}
		#endregion //ZoomOut_Click

		#endregion //RichTextEditorStudioUserControl Control Event Handlers

		#region XamRichTextEditor Event Handlers

		#region xamRichTextEditor1_DocumentChanged
		void xamRichTextEditor1_DocumentChanged(object sender, Infragistics.Controls.Editors.DocumentChangedEventArgs e)
		{
			if (e.OldDocument != null)
			{
			}

			this.HasCurrentDocument = (null != e.NewDocument);

			if (e.NewDocument != null)
			{
			}

			this.UpdateSettingsProperties();

			this.HasUnsavedChanges = false;
		}
		#endregion //xamRichTextEditor1_DocumentChanged

		#region xamRichTextEditor1_DocumentContentChanged
		void xamRichTextEditor1_DocumentContentChanged(object sender, DocumentContentChangedEventArgs e)
		{
			this.HasUnsavedChanges = true;
			this.UpdateSettingsProperties();

			this.RaiseDocumentContentOrSelectionChanged(EventArgs.Empty);
		}
		#endregion //xamRichTextEditor1_DocumentContentChanged

		#region xamRichTextEditor1_SelectionChanged
		void xamRichTextEditor1_SelectionChanged(object sender, Infragistics.Controls.Editors.RichTextSelectionChangedEventArgs e)
		{
			this.UpdateSettingsProperties();

			this.RaiseDocumentContentOrSelectionChanged(EventArgs.Empty);
		}
		#endregion //xamRichTextEditor1_SelectionChanged

		#endregion //XamRichTextEditor Event Handlers

		#endregion //Event Handlers

		#region Events

		#region DocumentContentOrSelectionChanged
		internal void RaiseDocumentContentOrSelectionChanged(EventArgs args)
		{
			EventHandler<EventArgs> handler = this.DocumentContentOrSelectionChanged;
			if (null != handler)
				handler(this, args);
		}

		public event EventHandler<EventArgs> DocumentContentOrSelectionChanged;
		#endregion //DocumentContentOrSelectionChanged

		#endregion //Events

		#region INotifyPropertyChanged Members

		public event PropertyChangedEventHandler PropertyChanged;

		#endregion

		#region DocumentFormatInfo Nested Private Class
		/// <summary>
		/// Class used to store information about a document format (and associated file extension) supported by the sample.
		/// </summary>
		private class DocumentFormatInfo
		{
			internal DocumentFormatInfo(SerializationProviderBase serializationProvider, string fileExtension, string description)
			{
				this.SerializationProvider	= serializationProvider;
				this.FileExtension			= fileExtension;
				this.Description			= description;
			}

			internal SerializationProviderBase	SerializationProvider { get; set; }
			internal string						FileExtension { get; set; }
			internal string						Description { get; set; }
		}
		#endregion //DocumentFormatInfo Nested Private Class
	}

	#region VisibilityToBoolConverter
	/// <summary>
	/// Represents the converter that converts Visibility enumeration values to and from bool values.
	/// </summary>
	public class VisibilityToBoolConverter : ValueConverterBase
	{
		#region IValueConverter Members

		/// <summary>
		/// Modifies the source data before passing it to the target for display in the UI.
		/// </summary>
		/// <param name="value">The source data being passed to the target.</param>
		/// <param name="targetType">The <see cref="T:System.Type"/> of data expected by the target dependency property.</param>
		/// <param name="parameter">An optional parameter to be used in the converter logic.</param>
		/// <param name="culture">The culture of the conversion.</param>
		/// <returns>
		/// The value to be passed to the target dependency property.
		/// </returns>
		public override object Convert(object value, System.Type targetType, object parameter, System.Globalization.CultureInfo culture)
		{
			Visibility valueResolved = Visibility.Collapsed;
			if (value is Visibility)
				valueResolved = (Visibility)value;
			else if (value is Visibility?)
			{
				Visibility? nullable = (Visibility?)value;
				valueResolved = nullable.HasValue ? nullable.Value : Visibility.Collapsed;
			}

			return valueResolved;
		}

		/// <summary>
		/// Modifies the target data before passing it to the source object.  This method is called only in <see cref="F:System.Windows.Data.BindingMode.TwoWay"/> bindings.
		/// </summary>
		/// <param name="value">The target data being passed to the source.</param>
		/// <param name="targetType">The <see cref="T:System.Type"/> of data expected by the source object.</param>
		/// <param name="parameter">An optional parameter to be used in the converter logic.</param>
		/// <param name="culture">The culture of the conversion.</param>
		/// <returns>
		/// The value to be passed to the source object.
		/// </returns>
		public override object ConvertBack(object value, System.Type targetType, object parameter, System.Globalization.CultureInfo culture)
		{
			if (value is bool)
			{
				return (bool)value == true ? Visibility.Visible : Visibility.Collapsed;
			}
			else
			{
				return Visibility.Collapsed;
			}
		}

		#endregion
	}
	#endregion //VisibilityToBoolConverter
}
